@@ -1,6 +1,6 @@
 """
-Module for reading and writing CPHD files. Support reading CPHD version 0.3 and 1.0
-and writing version 1.0.
+Module for reading and writing CPHD files. Support reading CPHD version 0.3 and 1
+and writing version 1.
 """
 
 __classification__ = "UNCLASSIFIED"
@@ -22,11 +22,11 @@ from sarpy.io.general.format_function import ComplexFormatFunction
 from sarpy.io.general.slice_parsing import verify_subscript, verify_slice
 
 from sarpy.io.phase_history.base import CPHDTypeReader
-# noinspection PyProtectedMember
-from sarpy.io.phase_history.cphd1_elements.CPHD import CPHDType as CPHDType1_0, \
-    CPHDHeader as CPHDHeader1_0, _CPHD_SECTION_TERMINATOR
+from sarpy.io.phase_history.cphd1_elements.CPHD import CPHDType as CPHDType1, \
+    CPHDHeader as CPHDHeader1, CPHD_SECTION_TERMINATOR
 from sarpy.io.phase_history.cphd0_3_elements.CPHD import CPHDType as CPHDType0_3, \
     CPHDHeader as CPHDHeader0_3
+from sarpy.io.phase_history.cphd_schema import get_namespace, get_default_tuple
 
 logger = logging.getLogger(__name__)
 
@@ -92,6 +92,18 @@ class AmpScalingFunction(ComplexFormatFunction):
     def set_amplitude_scaling(
             self,
             array: Optional[numpy.ndarray]) -> None:
+        """
+        Set the amplitude scaling array.
+
+        Parameters
+        ----------
+        array : None|numpy.ndarray
+
+        Returns
+        -------
+        None
+        """
+
         if array is None:
             self._amplitude_scaling = None
             return
@@ -235,17 +247,17 @@ class CPHDDetails(object):
         return self._cphd_version
 
     @property
-    def cphd_meta(self) -> Union[CPHDType1_0, CPHDType0_3]:
+    def cphd_meta(self) -> Union[CPHDType1, CPHDType0_3]:
         """
-        CPHDType1_0|CPHDType0_3: The CPHD metadata object, which is version dependent.
+        CPHDType1|CPHDType0_3: The CPHD metadata object, which is version dependent.
         """
 
         return self._cphd_meta
 
     @property
-    def cphd_header(self) -> Union[CPHDHeader1_0, CPHDHeader0_3]:
+    def cphd_header(self) -> Union[CPHDHeader1, CPHDHeader0_3]:
         """
-        CPHDHeader1_0|CPHDHeader0_3: The CPHD header object, which is version dependent.
+        CPHDHeader1|CPHDHeader0_3: The CPHD header object, which is version dependent.
         """
 
         return self._cphd_header
@@ -273,8 +285,8 @@ class CPHDDetails(object):
 
         if self.cphd_version.startswith('0.3'):
             self._cphd_header = CPHDHeader0_3.from_file_object(self._file_object)
-        elif self.cphd_version.startswith('1.0'):
-            self._cphd_header = CPHDHeader1_0.from_file_object(self._file_object)
+        elif self.cphd_version.startswith('1.'):
+            self._cphd_header = CPHDHeader1.from_file_object(self._file_object)
         else:
             raise ValueError(_unhandled_version_text.format(self.cphd_version))
 
@@ -286,8 +298,8 @@ class CPHDDetails(object):
         xml = self.get_cphd_bytes()
         if self.cphd_version.startswith('0.3'):
             the_type = CPHDType0_3
-        elif self.cphd_version.startswith('1.0'):
-            the_type = CPHDType1_0
+        elif self.cphd_version.startswith('1.'):
+            the_type = CPHDType1
         else:
             raise ValueError(_unhandled_version_text.format(self.cphd_version))
 
@@ -311,8 +323,8 @@ class CPHDDetails(object):
             # extract the xml data
             self.file_object.seek(header.XML_BYTE_OFFSET, os.SEEK_SET)
             xml = self.file_object.read(header.XML_DATA_SIZE)
-        elif self.cphd_version.startswith('1.0'):
-            assert isinstance(header, CPHDHeader1_0)
+        elif self.cphd_version.startswith('1.'):
+            assert isinstance(header, CPHDHeader1)
             # extract the xml data
             self.file_object.seek(header.XML_BLOCK_BYTE_OFFSET, os.SEEK_SET)
             xml = self.file_object.read(header.XML_BLOCK_SIZE)
@@ -334,15 +346,16 @@ class CPHDDetails(object):
         self.close()
 
 
-def _validate_cphd_details(cphd_details: Union[str, CPHDDetails],
-                           version: Optional[str]=None) -> CPHDDetails:
+def _validate_cphd_details(
+        cphd_details: Union[str, CPHDDetails],
+        version: Union[None, str, Sequence[str]] = None) -> CPHDDetails:
     """
     Validate the input argument.
 
     Parameters
     ----------
     cphd_details : str|CPHDDetails
-    version : None|str
+    version : None|str|Sequence[str]
 
     Returns
     -------
@@ -362,10 +375,22 @@ def _validate_cphd_details(cphd_details: Union[str, CPHDDetails],
     if not isinstance(cphd_details, CPHDDetails):
         raise TypeError('cphd_details is required to be a file path to a CPHD file '
                         'or CPHDDetails, got type {}'.format(cphd_details))
+    if version is not None:
+        if isinstance(version, str) and not cphd_details.cphd_version.startswith(version):
+            raise ValueError(
+                'This CPHD file is required to be version {},\n\t'
+                'got {}'.format(version, cphd_details.cphd_version))
+        else:
+            val = False
+            for entry in version:
+                if cphd_details.cphd_version.startswith(entry):
+                    val = True
+                    break
+            if not val:
+                raise ValueError(
+                    'This CPHD file is required to be one of version {},\n\t'
+                    'got {}'.format(version, cphd_details.cphd_version))
 
-    if version is not None and not cphd_details.cphd_version.startswith(version):
-        raise ValueError('This CPHD file is required to be version {}, '
-                         'got {}'.format(version, cphd_details.cphd_version))
     return cphd_details
 
 
@@ -396,8 +421,8 @@ class CPHDReader(CPHDTypeReader):
 
         if cphd_details.cphd_version.startswith('0.3'):
             return object.__new__(CPHDReader0_3)
-        elif cphd_details.cphd_version.startswith('1.0'):
-            return object.__new__(CPHDReader1_0)
+        elif cphd_details.cphd_version.startswith('1.'):
+            return object.__new__(CPHDReader1)
         else:
             raise ValueError('Got unhandled CPHD version {}'.format(cphd_details.cphd_version))
 
@@ -418,9 +443,9 @@ class CPHDReader(CPHDTypeReader):
         return self.cphd_details.cphd_version
 
     @property
-    def cphd_header(self) -> Union[CPHDHeader1_0, CPHDHeader0_3]:
+    def cphd_header(self) -> Union[CPHDHeader1, CPHDHeader0_3]:
         """
-        CPHDHeader1_0|CPHDHeader0_3: The CPHD header object, which is version dependent.
+        CPHDHeader1|CPHDHeader0_3: The CPHD header object, which is version dependent.
         """
 
         return self.cphd_details.cphd_header
@@ -433,13 +458,13 @@ class CPHDReader(CPHDTypeReader):
             self,
             variable: str,
             index: Union[int, str],
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> Optional[numpy.ndarray]:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> Optional[numpy.ndarray]:
         raise NotImplementedError
 
     def read_pvp_array(
             self,
             index: Union[int, str],
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> numpy.ndarray:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> numpy.ndarray:
         raise NotImplementedError
 
     def read_pvp_block(self) -> Dict[Union[int, str], numpy.ndarray]:
@@ -459,12 +484,13 @@ class CPHDReader(CPHDTypeReader):
             del self._cphd_details
 
 
-class CPHDReader1_0(CPHDReader):
+class CPHDReader1(CPHDReader):
     """
-    The CPHD version 1.0 reader.
+    The CPHD version 1 reader.
 
     **Updated in version 1.3.0** for reading changes.
     """
+    _allowed_versions = ('1.0', '1.1')
 
     def __new__(cls, *args, **kwargs):
         # we must override here, to avoid recursion with
@@ -482,7 +508,7 @@ class CPHDReader1_0(CPHDReader):
         self._channel_map = None  # type: Union[None, Dict[str, int]]
         self._pvp_memmap = None  # type: Union[None, Dict[str, numpy.ndarray]]
         self._support_array_memmap = None  # type: Union[None, Dict[str, numpy.ndarray]]
-        self._cphd_details = _validate_cphd_details(cphd_details, version='1.0')
+        self._cphd_details = _validate_cphd_details(cphd_details, version=self._allowed_versions)
 
         CPHDTypeReader.__init__(self, None, self._cphd_details.cphd_meta)
         # set data segments after setting up the pvp information, because
@@ -494,17 +520,17 @@ class CPHDReader1_0(CPHDReader):
         BaseReader.__init__(self, data_segments, reader_type='CPHD')
 
     @property
-    def cphd_meta(self) -> CPHDType1_0:
+    def cphd_meta(self) -> CPHDType1:
         """
-        CPHDType1_0: The CPHD structure.
+        CPHDType1: The CPHD structure.
         """
 
         return self._cphd_meta
 
     @property
-    def cphd_header(self) -> CPHDHeader1_0:
+    def cphd_header(self) -> CPHDHeader1:
         """
-        CPHDHeader1_0: The CPHD header object.
+        CPHDHeader1: The CPHD header object.
         """
 
         return self.cphd_details.cphd_header
@@ -650,9 +676,10 @@ class CPHDReader1_0(CPHDReader):
                 raise ValueError(_index_range_text.format(cphd_meta.Data.NumCPHDChannels))
             return cphd_meta.Data.Channels[int_index].Identifier
 
-    def read_support_array(self,
-                           index: Union[int, str],
-                           *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]]) -> numpy.ndarray:
+    def read_support_array(
+            self,
+            index: Union[int, str],
+            *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]]) -> numpy.ndarray:
         # find the support array identifier
         if isinstance(index, int):
             the_entry = self.cphd_meta.Data.SupportArrays[index]
@@ -681,7 +708,7 @@ class CPHDReader1_0(CPHDReader):
             self,
             variable: str,
             index: Union[int, str],
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> Optional[numpy.ndarray]:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> Optional[numpy.ndarray]:
         index_key = self._validate_index_key(index)
         the_memmap = self._pvp_memmap[index_key]
         the_slice = verify_slice(the_range, the_memmap.shape[0])
@@ -693,25 +720,29 @@ class CPHDReader1_0(CPHDReader):
     def read_pvp_array(
             self,
             index: Union[int, str],
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> numpy.ndarray:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> numpy.ndarray:
         index_key = self._validate_index_key(index)
         the_memmap = self._pvp_memmap[index_key]
         the_slice = verify_slice(the_range, the_memmap.shape[0])
         return numpy.copy(the_memmap[the_slice])
 
     def read_pvp_block(self) -> Dict[str, numpy.ndarray]:
-        return {chan.Identifier: self.read_pvp_array(chan.Identifier) for chan in self.cphd_meta.Data.Channels}
+        return {chan.Identifier: self.read_pvp_array(chan.Identifier)
+                for chan in self.cphd_meta.Data.Channels}
 
     def read_signal_block(self) -> Dict[str, numpy.ndarray]:
-        return {chan.Identifier: numpy.copy(self.read(index=chan.Identifier)) for chan in self.cphd_meta.Data.Channels}
+        return {chan.Identifier: numpy.copy(self.read(index=chan.Identifier))
+                for chan in self.cphd_meta.Data.Channels}
 
     def read_signal_block_raw(self) -> Dict[Union[int, str], numpy.ndarray]:
-        return {chan.Identifier: numpy.copy(self.read_raw(index=chan.Identifier)) for chan in self.cphd_meta.Data.Channels}
+        return {chan.Identifier: numpy.copy(self.read_raw(index=chan.Identifier))
+                for chan in self.cphd_meta.Data.Channels}
 
-    def read_chip(self,
-             *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
-             index: Union[int, str]=0,
-             squeeze: bool=True) -> numpy.ndarray:
+    def read_chip(
+            self,
+            *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
+            index: Union[int, str] = 0,
+            squeeze: bool = True) -> numpy.ndarray:
         """
         This is identical to :meth:`read`, and presented for backwards compatibility.
 
@@ -732,10 +763,11 @@ class CPHDReader1_0(CPHDReader):
 
         return self.__call__(*ranges, index=index, raw=False, squeeze=squeeze)
 
-    def read(self,
-             *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
-             index: Union[int, str]=0,
-             squeeze: bool=True) -> numpy.ndarray:
+    def read(
+            self,
+            *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
+            index: Union[int, str] = 0,
+            squeeze: bool = True) -> numpy.ndarray:
         """
         Read formatted data from the given data segment. Note this is an alias to the
         :meth:`__call__` called as
@@ -761,10 +793,11 @@ class CPHDReader1_0(CPHDReader):
 
         return self.__call__(*ranges, index=index, raw=False, squeeze=squeeze)
 
-    def read_raw(self,
-                 *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
-                 index: Union[int, str]=0,
-                 squeeze: bool=True) -> numpy.ndarray:
+    def read_raw(
+            self,
+            *ranges: Sequence[Union[None, int, Tuple[int, ...], slice]],
+            index: Union[int, str] = 0,
+            squeeze: bool = True) -> numpy.ndarray:
         """
         Read raw data from the given data segment. Note this is an alias to the
         :meth:`__call__` called as
@@ -790,11 +823,12 @@ class CPHDReader1_0(CPHDReader):
 
         return self.__call__(*ranges, index=index, raw=True, squeeze=squeeze)
 
-    def __call__(self,
-                 *ranges: Sequence[Union[None, int, slice]],
-                 index: int=0,
-                 raw: bool=False,
-                 squeeze: bool=True) -> numpy.ndarray:
+    def __call__(
+            self,
+            *ranges: Sequence[Union[None, int, slice]],
+            index: int = 0,
+            raw: bool = False,
+            squeeze: bool = True) -> numpy.ndarray:
         index = self._validate_index(index)
         return BaseReader.__call__(self, *ranges, index=index, raw=raw, squeeze=squeeze)
 
@@ -910,7 +944,7 @@ class CPHDReader0_3(CPHDReader):
             self,
             variable: str,
             index: int,
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> Optional[numpy.ndarray]:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> Optional[numpy.ndarray]:
         int_index = self._validate_index(index)
         the_memmap = self._pvp_memmap[int_index]
         the_slice = verify_slice(the_range, the_memmap.shape[0])
@@ -922,7 +956,7 @@ class CPHDReader0_3(CPHDReader):
     def read_pvp_array(
             self,
             index: int,
-            the_range: Union[None, int, Tuple[int, ...], slice]=None) -> numpy.ndarray:
+            the_range: Union[None, int, Tuple[int, ...], slice] = None) -> numpy.ndarray:
         int_index = self._validate_index(index)
         the_memmap = self._pvp_memmap[int_index]
         the_slice = verify_slice(the_range, the_memmap.shape[0])
@@ -946,11 +980,12 @@ class CPHDReader0_3(CPHDReader):
     def read_signal_block_raw(self) -> Dict[int, numpy.ndarray]:
         return {chan: self.read_raw(index=chan) for chan in range(self.cphd_meta.Data.NumCPHDChannels)}
 
-    def __call__(self,
-                 *ranges: Sequence[Union[None, int, slice]],
-                 index: int=0,
-                 raw: bool=False,
-                 squeeze: bool=True) -> numpy.ndarray:
+    def __call__(
+            self,
+            *ranges: Sequence[Union[None, int, slice]],
+            index: int = 0,
+            raw: bool = False,
+            squeeze: bool = True) -> numpy.ndarray:
         index = self._validate_index(index)
         return BaseReader.__call__(self, *ranges, index=index, raw=raw, squeeze=squeeze)
 
@@ -1003,7 +1038,7 @@ class ElementDetails(object):
         return self._item_offset
 
     @item_offset.setter
-    def item_offset(self, value) -> None:
+    def item_offset(self, value: int) -> None:
         value = int(value)
         if self._item_offset is not None and self._item_offset != value:
             raise ValueError("item_offset is read only after being initially defined.")
@@ -1038,7 +1073,7 @@ class ElementDetails(object):
         return self._item_written
 
     @item_written.setter
-    def item_written(self, value):
+    def item_written(self, value: bool):
         value = bool(value)
         if self._item_written and not value:
             raise ValueError(
@@ -1082,7 +1117,7 @@ class CPHDWritingDetails(object):
         '_channel_map', '_support_map',
         '_pvp_details', '_support_details', '_signal_details')
 
-    def __init__(self, meta: CPHDType1_0):
+    def __init__(self, meta: CPHDType1, check_older_version: bool = False):
 
         self._header = None
         self._header_written = False
@@ -1094,7 +1129,7 @@ class CPHDWritingDetails(object):
         self._signal_details = None
 
         self.meta = meta
-        self._header = self.meta.make_file_header()
+        self._set_header(check_older_version)
 
         # initialize the information for the pvp, support, and signal details
         self._populate_pvp_details()
@@ -1102,13 +1137,25 @@ class CPHDWritingDetails(object):
         self._populate_signal_details()
 
     @property
-    def header(self) -> CPHDHeader1_0:
+    def header(self) -> CPHDHeader1:
         return self._header
 
+    def _set_header(self, check_older_version: bool):
+        if check_older_version:
+            use_version_tuple = self.meta.version_required()
+        else:
+            use_version_tuple = get_default_tuple()
+        use_version_string = '{}.{}.{}'.format(*use_version_tuple)
+        self._header = self.meta.make_file_header(use_version=use_version_string)
+
     @property
-    def meta(self) -> CPHDType1_0:
+    def use_version(self) -> str:
+        return self.header.use_version
+
+    @property
+    def meta(self) -> CPHDType1:
         """
-        CPHDType1_0: The metadata
+        CPHDType1: The metadata
         """
 
         return self._meta
@@ -1117,8 +1164,8 @@ class CPHDWritingDetails(object):
     def meta(self, value):
         if self._meta is not None:
             raise ValueError('meta is read only once initialized.')
-        if not isinstance(value, CPHDType1_0):
-            raise TypeError('meta must be of type {}'.format(CPHDType1_0))
+        if not isinstance(value, CPHDType1):
+            raise TypeError('meta must be of type {}'.format(CPHDType1))
         self._meta = value
 
     def _populate_pvp_details(self) -> None:
@@ -1176,13 +1223,19 @@ class CPHDWritingDetails(object):
     def support_map(self) -> Optional[Dict[str, int]]:
         return self._support_map
 
-    def _write_items(self, details: Optional[Sequence[ElementDetails]], file_object: BinaryIO) -> None:
+    def _write_items(
+            self,
+            details: Optional[Sequence[ElementDetails]],
+            file_object: BinaryIO) -> None:
         if details is None:
             return
         for index, entry in enumerate(details):
             entry.write_item(file_object)
 
-    def _verify_item_written(self, details: Optional[Sequence[ElementDetails]], name: str) -> None:
+    def _verify_item_written(
+            self,
+            details: Optional[Sequence[ElementDetails]],
+            name: str) -> None:
         if details is None:
             return
 
@@ -1190,7 +1243,10 @@ class CPHDWritingDetails(object):
             if not entry.item_written:
                 logger.error('{} data at index {} not written'.format(name, index))
 
-    def write_header(self, file_object: BinaryIO, overwrite: bool = False) -> None:
+    def write_header(
+            self,
+            file_object: BinaryIO,
+            overwrite: bool = False) -> None:
         """
         Write the header.The file object will be advanced to the end of the
         block, if writing occurs.
@@ -1210,11 +1266,11 @@ class CPHDWritingDetails(object):
             return
 
         file_object.write(self.header.to_string().encode())
-        file_object.write(_CPHD_SECTION_TERMINATOR)
+        file_object.write(CPHD_SECTION_TERMINATOR)
         # write xml
         file_object.seek(self.header.XML_BLOCK_BYTE_OFFSET, os.SEEK_SET)
-        file_object.write(self.meta.to_xml_bytes())
-        file_object.write(_CPHD_SECTION_TERMINATOR)
+        file_object.write(self.meta.to_xml_bytes(urn=get_namespace(self.use_version)))
+        file_object.write(CPHD_SECTION_TERMINATOR)
         self._header_written = True
 
     def write_all_populated_items(self, file_object: BinaryIO) -> None:
@@ -1245,9 +1301,9 @@ class CPHDWritingDetails(object):
         self._verify_item_written(self.signal_details, 'signal')
 
 
-class CPHDWriter1_0(BaseWriter):
+class CPHDWriter1(BaseWriter):
     """
-    The CPHD version 1.0 writer.
+    The CPHD version 1 writer.
 
     **Updated in version 1.3.0** for writing changes.
     """
@@ -1258,18 +1314,23 @@ class CPHDWriter1_0(BaseWriter):
         '_pvp_memmaps', '_support_memmaps', '_signal_data_segments',
         '_can_write_regular_data')
 
-    def __init__(self,
-                 file_object: Union[str, BinaryIO],
-                 meta: Optional[CPHDType1_0] = None,
-                 writing_details: Optional[CPHDWritingDetails] = None,
-                 check_existence: bool=True):
+    def __init__(
+            self,
+            file_object: Union[str, BinaryIO],
+            meta: Optional[CPHDType1] = None,
+            writing_details: Optional[CPHDWritingDetails] = None,
+            check_older_version: bool = False,
+            check_existence: bool = True):
         """
 
         Parameters
         ----------
         file_object : str|BinaryIO
-        meta : None|CPHDType1_0
+        meta : None|CPHDType1
         writing_details : None|CPHDWritingDetails
+        check_older_version : bool
+            Try to create an older version CPHD for compliance with other
+            NGA applications
         check_existence : bool
             Should we check if the given file already exists, and raises an exception if so?
         """
@@ -1279,7 +1340,7 @@ class CPHDWriter1_0(BaseWriter):
         if isinstance(file_object, str):
             if check_existence and os.path.exists(file_object):
                 raise SarpyIOError(
-                    'Given file {} already exists, and a new NITF file cannot be created here.'.format(file_object))
+                    'Given file {} already exists, and a new CPHD file cannot be created here.'.format(file_object))
             file_object = open(file_object, 'wb')
 
         if not is_file_like(file_object):
@@ -1296,7 +1357,7 @@ class CPHDWriter1_0(BaseWriter):
         if meta is None and writing_details is None:
             raise ValueError('One of meta or writing_details must be provided.')
         if writing_details is None:
-            writing_details = self._writing_details_type(meta)
+            writing_details = self._writing_details_type(meta, check_older_version=check_older_version)
         self.writing_details = writing_details
 
         self._pvp_memmaps = None  # type: Optional[Dict[str, numpy.ndarray]]
@@ -1306,7 +1367,6 @@ class CPHDWriter1_0(BaseWriter):
         self._closed = False
 
         data_segment = self._initialize_data()
-        # data_segment = self._prepare_for_writing()
         BaseWriter.__init__(self, data_segment)
 
     @property
@@ -1326,9 +1386,9 @@ class CPHDWriter1_0(BaseWriter):
         return self._file_name
 
     @property
-    def meta(self) -> CPHDType1_0:
+    def meta(self) -> CPHDType1:
         """
-        CPHDType1_0: The metadata
+        CPHDType1: The metadata
         """
 
         return self.writing_details.meta
@@ -1666,10 +1726,11 @@ class CPHDWriter1_0(BaseWriter):
         for identifier, array in signal_block.items():
             self.write_raw(array, index=identifier)
 
-    def write_file(self,
-                   pvp_block: Dict[Union[int, str], numpy.ndarray],
-                   signal_block: Dict[Union[int, str], numpy.ndarray],
-                   support_block: Optional[Dict[Union[int, str], numpy.ndarray]]=None):
+    def write_file(
+            self,
+            pvp_block: Dict[Union[int, str], numpy.ndarray],
+            signal_block: Dict[Union[int, str], numpy.ndarray],
+            support_block: Optional[Dict[Union[int, str], numpy.ndarray]] = None):
         """
         Write the blocks to the file.
 
@@ -1691,10 +1752,11 @@ class CPHDWriter1_0(BaseWriter):
             self.write_support_block(support_block)
         self.write_signal_block(signal_block)
 
-    def write_file_raw(self,
-                   pvp_block: Dict[Union[int, str], numpy.ndarray],
-                   signal_block: Dict[Union[int, str], numpy.ndarray],
-                   support_block: Optional[Dict[Union[int, str], numpy.ndarray]]=None):
+    def write_file_raw(
+            self,
+            pvp_block: Dict[Union[int, str], numpy.ndarray],
+            signal_block: Dict[Union[int, str], numpy.ndarray],
+            support_block: Optional[Dict[Union[int, str], numpy.ndarray]] = None):
         """
         Write the blocks to the file.
 
@@ -1716,33 +1778,37 @@ class CPHDWriter1_0(BaseWriter):
             self.write_support_block(support_block)
         self.write_signal_block_raw(signal_block)
 
-    def write_chip(self,
-              data: numpy.ndarray,
-              start_indices: Union[None, int, Tuple[int, ...]] = None,
-              subscript: Union[None, Tuple[slice, ...]] = None,
-              index: Union[int, str]=0) -> None:
+    def write_chip(
+            self,
+            data: numpy.ndarray,
+            start_indices: Union[None, int, Tuple[int, ...]] = None,
+            subscript: Union[None, Tuple[slice, ...]] = None,
+            index: Union[int, str] = 0) -> None:
         self.__call__(data, start_indices=start_indices, subscript=subscript, index=index, raw=False)
 
-    def write(self,
-              data: numpy.ndarray,
-              start_indices: Union[None, int, Tuple[int, ...]] = None,
-              subscript: Union[None, Tuple[slice, ...]] = None,
-              index: Union[int, str]=0) -> None:
+    def write(
+            self,
+            data: numpy.ndarray,
+            start_indices: Union[None, int, Tuple[int, ...]] = None,
+            subscript: Union[None, Tuple[slice, ...]] = None,
+            index: Union[int, str] = 0) -> None:
         self.__call__(data, start_indices=start_indices, subscript=subscript, index=index, raw=False)
 
-    def write_raw(self,
-              data: numpy.ndarray,
-              start_indices: Union[None, int, Tuple[int, ...]]=None,
-              subscript: Union[None, Tuple[slice, ...]]=None,
-              index: Union[int, str]=0) -> None:
+    def write_raw(
+            self,
+            data: numpy.ndarray,
+            start_indices: Union[None, int, Tuple[int, ...]] = None,
+            subscript: Union[None, Tuple[slice, ...]] = None,
+            index: Union[int, str] = 0) -> None:
         self.__call__(data, start_indices=start_indices, subscript=subscript, index=index, raw=False)
 
-    def __call__(self,
-                 data: numpy.ndarray,
-                 start_indices: Union[None, int, Tuple[int, ...]]=None,
-                 subscript: Union[None, Tuple[slice, ...]]=None,
-                 index: Union[int, str]=0,
-                 raw: bool=False) -> None:
+    def __call__(
+            self,
+            data: numpy.ndarray,
+            start_indices: Union[None, int, Tuple[int, ...]] = None,
+            subscript: Union[None, Tuple[slice, ...]] = None,
+            index: Union[int, str] = 0,
+            raw: bool = False) -> None:
         int_index = self._validate_channel_index(index)
 
         identifier = self._validate_channel_key(index)
